/**
  *  \file output.c
  */

#include <stdio.h>
#include <stdlib.h>
#include "output.h"
#include "recog.h"
#include "util.h"
#include "ringbuf.h"

/********************************* Output ********************************/

static FILE* output_fp;

void
output_open(M* m)
{
    if (!m->rec_name) {
        static int counter;
        char* ext = m->rec_ext ? m->rec_ext : "dat";
        char* all = xmalloc(strlen(ext) + 20);
        sprintf(all, "file%04d.%s", ++counter, ext);
        match_set_name(m, all);
        free(all);
    }
    output_fp = fopen(m->rec_name, "wb");
    if (!output_fp) {
        perror(m->rec_name);
    } else {
        if (m->reader)
            printf("Extracting %s (stream)...\n", m->rec_name);
        else 
            printf("Extracting %s (%ld bytes)...\n", m->rec_name, (long)m->length);
    }
}

void
output_close(M* m)
{
    ARG_USED(m);
    if (output_fp)
        fclose(output_fp);
    output_fp = 0;
}

void
output_write(M* m, char* data, size_t len)
{
    if (output_fp)
        if (fwrite(data, 1, len, output_fp) != len)
            fprintf(stderr, "error writing to %s\n", m->rec_name);
}

void
output_copy(M* m, BUF* b, length_t len)
{
    static char buffer[BUFFER_SIZE];
    while (len > 0) {
        size_t now = (len > BUFFER_SIZE ? BUFFER_SIZE : len);
        size_t got = buf_read(b, buffer, now);
        if (!got) {
            fprintf(stderr, "input ends prematurely.\n");
            return;
        }
        len -= got;
        output_write(m, buffer, got);
    }
}
