/**
  *  \file recog.c
  */

#include <limits.h>
#include <stdlib.h>
#include "recog.h"
#include "util.h"

struct RecognizerList* recog_list[UCHAR_MAX];

void
recog_add(REC* r)
{
    const char* i = r->firsts;
    if (i) {
        do {
            unsigned char here = *i;
            struct RecognizerList* node = xmalloc(sizeof(struct RecognizerList));
            node->next = recog_list[here];
            node->recog = r;
            recog_list[here] = node;
        } while (*++i);
    } else {
        int i;
        for (i = 0; i <= UCHAR_MAX; ++i) {
            struct RecognizerList* node = xmalloc(sizeof(struct RecognizerList));
            node->next = recog_list[i];
            node->recog = r;
            recog_list[i] = node;
        }
    }
}


void
match_set_ext(M* m, const char* c)
{
    set(&m->rec_ext, dup(c));
}

void
match_set_name(M* m, const char* fn)
{
    set(&m->rec_name, dup(fn));
}

void
match_set_length(M* m, length_t len)
{
    m->length = len;
}

void
match_set_reader(M* m, void (*r)(BUF*, M*))
{
    m->reader = r;
}

void
match_init(M* m)
{
    m->rec_name = m->rec_ext = 0;
    m->length = 0;
    m->reader = 0;
}

void
match_done(M* m)
{
    if (m->rec_ext)
        free(m->rec_ext);
    if (m->rec_name)
        free(m->rec_name);
    match_init(m);
}
