/**
  *  \file recog.h
  */
#ifndef RECOG_H_INCLUDED
#define RECOG_H_INCLUDED

#include "ringbuf.h"

typedef long length_t;

struct Match;

struct Recognizer {
    const char*   id;             /* identification */
    const char*   firsts;         /* first bytes */
    int         (*recog)(struct Match*, BUF*, size_t);
};
typedef struct Recognizer REC;

struct RecognizerList {
    struct RecognizerList* next;
    struct Recognizer*     recog;
};

struct Match {
    char*    rec_ext;
    char*    rec_name;
    length_t length;
    void   (*reader)(BUF*, struct Match* m);
};
typedef struct Match M;

extern struct RecognizerList* recog_list[];

void     recog_add(REC* r);
#define  recog_entry(N) (recog_list[(unsigned char)(N)])

void     match_set_ext(M*, const char* c);
void     match_set_name(M*, const char* fn);
void     match_set_length(M*, length_t len);
void     match_set_reader(M*, void (*)(BUF*, M*));
void     match_init(M*);
void     match_done(M*);

#endif
