/**
  *  \file recog_png.c
  */

#include "recog.h"

int
recog_png(M* m, BUF* b, size_t len)
{
    unsigned long length;
    if (len < 40 || (unsigned char) buf_peek(b, 0) != 137
        || buf_peek(b, 1) != 80 || buf_peek(b, 2) != 78
        || buf_peek(b, 3) != 71 || buf_peek(b, 4) != 13
        || buf_peek(b, 5) != 10 || buf_peek(b, 6) != 26
        || buf_peek(b, 7) != 10
        || get_b32(b, 12) != 0x49484452 /* IHDR */)
        return 0;
    /* chunks: 4 byte BE   length
               4 byte      sig
               --          data
               4 byte      CRC */
    length = 8;
    while (length + 12 < len) {
        unsigned long clen  = get_b32(b, length);
        unsigned long ctype = get_b32(b, length+4);
        if (clen > 0x80000000U || clen > len || length + 12 + clen > len)
            return 0;
        if (ctype & 0x80808080U)
            return 0;
        length += 12+clen;

        if (ctype == 0x49454E44) /* IEND */
            break;
    }
    if (length > 8) {
        match_set_length(m, length);
        match_set_ext(m, "png");
        return 1;
    }
    return 0;
}

REC png_recognizer = { "png", "\x89", recog_png };
