/**
  *  \file recog_riff.c
  *
  *  This recognizes RIFF files (wav/rmi/avi). Note that wav is sometimes
  *  used as an mp3 container.
  */

#include "recog.h"

int
recog_riff(M* m, BUF* b, size_t len)
{
    unsigned long  length, ftype;

    if (len < 50 || buf_peek(b, 0) != 'R' || buf_peek(b, 1) != 'I'
        || buf_peek(b, 2) != 'F' || buf_peek(b, 3) != 'F')
        return 0;
    length = get_l32(b, 4);
    ftype  = get_l32(b, 8);
    if (((ftype & 0x80808080UL) == 0) && (ftype & 0xFF000000UL)
        && (ftype & 0xFF0000UL) && (ftype & 0xFF00UL) && (ftype & 0xFF)
        && length > 0)
    {
        match_set_length(m, length + 8);
        switch(ftype) {
         case 0x45564157:       /* "WAVE" */
            match_set_ext(m, "wav");
            break;
         case 0x44494D52:       /* "RMID" */
            match_set_ext(m, "rmi");
            break;
         case 0x20495641:       /* "AVI " */
            match_set_ext(m, "avi");
            break;
         default:
            match_set_ext(m, "rif");
            break;
        }
        return 1;
    }
    return 0;
}

REC riff_recognizer = { "riff", "R", recog_riff };
