/**
  *  \file ringbuf.h
  */
#ifndef RINGBUF_H_INCLUDED
#define RINGBUF_H_INCLUDED

#include <stdio.h>
#include "defs.h"

struct Ringbuffer {
    char*   data;               /* data */
    size_t  size;               /* allocated size of data */
    size_t  length;             /* total length of data */
    size_t  pos;                /* position */
    FILE*   fp;
    bytecount_t total_read;
};
typedef struct Ringbuffer BUF;

void   buf_init(BUF* buf, FILE* fp, size_t size);
void   buf_free(BUF* buf);
void   buf_advance(BUF* buf, size_t by);
size_t buf_read(BUF* buf, char* data, size_t count);
size_t buf_avail_lin(BUF* buf);
int    buf_streq(BUF* buf, size_t pos, const char* str);
int    buf_memeq(BUF* buf, size_t pos, const char* data, size_t length);

#define buf_peek(buf, n) ((buf)->data[(n + (buf)->pos) % (buf)->size])
#define buf_peeku(buf, n) ((unsigned char)(buf_peek(buf, n)))

static inline unsigned short get_l16(BUF* b, size_t i) {
    return buf_peeku(b, i) + 256*buf_peeku(b, i+1);
}

static inline unsigned short get_b16(BUF* b, size_t i) {
    return buf_peeku(b, i+1) + 256*buf_peeku(b, i);
}

static inline unsigned long get_l32(BUF* b, size_t i) {
    return get_l16(b, i) + 65536UL * get_l16(b, i + 2);
}

static inline unsigned long get_b32(BUF* b, size_t i) {
    return get_b16(b, i + 2) + 65536UL * get_b16(b, i);
}

#endif
