/**
  *  \file ripper.c
  */

#include <stdlib.h>
#include <stdio.h>
#include <stdarg.h>
#include <limits.h>
#include <string.h>
#include "ripper.h"
#include "recog.h"
#include "util.h"
#include "ringbuf.h"
#include "output.h"

static char* arg_filename;

static unsigned int block_size = 1/*512*/;

/********************************* Ripper ********************************/

void
scan_buffer(BUF* buf)
{
    size_t                 index;
    struct RecognizerList* rl;
    M                      match;

    index = 0;
    match_init(&match);
    while (1) {
/*         printf("%lld %% %d == %lld\n", (buf->total_read + index), block_size, */
/*                ((buf->total_read + index) % block_size)); */
        assert(((buf->total_read + index) % block_size) == 0);
        if ((rl = recog_entry(buf_peek(buf, index))) != 0) {
            if (index)
                buf_advance(buf, index);
            index = 0;
            while (rl && !rl->recog->recog(&match, buf, buf->length))
                rl = rl->next;
            if (rl) {
                bytecount_t s = buf->total_read;
                output_open(&match);
                if (match.reader) {
                    match.reader(buf, &match);
                } else {
                    output_copy(&match, buf, match.length);
                }
                output_close(&match);
                match_done(&match);
                s = buf->total_read - s;
/*                 printf("        " BYTECOUNT_FMT " bytes copied\n", s); */
                if (!s)
                    index = block_size;
                else if (s % block_size)
                    index = block_size - (s % block_size);
                else
                    index = 0;
                continue;
            }
        }
        
        index += block_size;
        if (index >= buf->length) {
            index -= buf->length;
            buf_advance(buf, buf->length);
        }
        if (index >= buf->length)
            break;
    }
}

void
scan_stream(FILE* fp)
{
    BUF buffer;
    buf_init(&buffer, fp, BUFFER_SIZE);
    scan_buffer(&buffer);
    buf_free(&buffer);
}

void
scan_file(const char* filename)
{
    FILE* fp = fopen(filename, "rb");
    if (!fp) {
        perror(filename);
        return;
    }
    scan_stream(fp);
    fclose(fp);
}

/********************************** Main *********************************/

int
main(int argc, char** argv)
{
    ARG_USED(argc);
    arg_filename = argv[1];
    if (!arg_filename)
        die("No filename specified.");

    recog_add(&bmp_recognizer);
    recog_add(&mp3_recognizer);
    recog_add(&riff_recognizer);
    recog_add(&dir_recognizer);
    recog_add(&png_recognizer);
    scan_file(arg_filename);
    puts("");
    return 0;
}
