/**
  *  \file util.c
  */

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include "util.h"

static char* current_file;

void
set_current_file(const char* f)
{
    current_file = f ? dup(f) : 0;
}

void
die(const char* fmt, ...)
{
    va_list va;
    if (current_file)
        fprintf(stderr, "%s: ", current_file);
    va_start(va, fmt);
    vfprintf(stderr, fmt, va);
    fprintf(stderr, "\n");
    exit(1);
}

void*
xmalloc(size_t s)
{
    void* p = malloc(s);
    if (!p)
        die("out of memory");
    return p;
}

char*
dup(const char* s)
{
    size_t len = strlen(s) + 1;
    char*  p = xmalloc(len);
    memcpy(p, s, len);
    return p;
}

void
set(char** old, char* new_)
{
    if (*old)
        free(*old);
    *old = new_;
}

size_t
full_read(FILE* fp, char* buffer, size_t size)
{
    size_t total = 0;
    while (size) {
        size_t got = fread(buffer, 1, size, fp);
        if (!got)
            break;
        total += got;
        size  -= got;
        buffer += got;
    }
    return total;
}
